<?php
class Wp_Dxp_Admin_Rules_Page extends Wp_Dxp_Admin_Base_Page {

	/**
	 * Process - typically used for processing POST data
	 */
	public function process()
	{
		$action = $this->request->get('wp_dxp_action');
		if (!empty($this->request->form())) {
			switch ($action) {
				case 'create':
					return $this->storeAction();
				case 'edit':
					return $this->updateAction();
			}
		}

		switch ($action) {
			case 'delete':
				return $this->deleteAction();
			case 'duplicate':
				return $this->duplicateAction();
		}
	}

	/**
	 * Route to the correct action within the page
	 */
	public function route()
	{
		$action = $this->request->get('wp_dxp_action');

		switch ($action) {
			case 'create':
				return $this->createAction();
			case 'edit':
				return $this->editAction();
		}

		return $this->createAction();
	}

	/**
	 * Create action - display form
	 */
	public function createAction()
	{
		$rule = new Wp_Dxp_Rule();

		if ($data = $this->request->form()) {
			$rule->populateFromArray($data);
		}

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/rules/create.php';
	}

	/**
	 * Store action - process data from form submitted on "create" page
	 */
	public function storeAction()
	{
		if ($this->request->validateNonce(WP_DXP_FORM_NONCE_FIELD_ACTION)) {

			$data = $this->request->form();
			$this->validate($data);

			if (empty($this->getError())) {
				$this->saveRule($data);

				$this->addFlashMessage(__('Rule created', 'wp-dxp'), "success");

				$redirectUrl = site_url(WP_DXP_ADMIN_DASHBOARD_INDEX_URL);
				wp_redirect($redirectUrl);
				exit;
			}

			return;
		}

		$this->addFlashMessage(__('An error occurred, please reload the page', 'wp-dxp'), "error");
	}

	/**
	 * Edit action - display form for existing rule
	 */
	public function editAction()
	{
		$id = $this->request->get('id', false);

		$rule = Wp_Dxp_Rule::find($id);

		if (!$rule) {
			$this->showError(__('Rule could not be found', 'wp-dxp'));
		}

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/rules/edit.php';
	}

	/**
	 * Update action - process data from form submitted on "edit" page
	 */
	public function updateAction()
	{
		if ($this->request->validateNonce(WP_DXP_FORM_NONCE_FIELD_ACTION)) {

			$id = $this->request->form('id', false);

			$rule = Wp_Dxp_Rule::find($id);

			if (!$rule) {
				$this->showError(__('Rule could not be found', 'wp-dxp'));
			}

			if (!$rule->can_edit) {
				$this->showError(__('This rule cannot be edited', 'wp-dxp'));
			}

			$data = $this->request->form();

			$this->validate($data, true);

			if (empty($this->getError())) {
				$this->saveRule($data, $id);

				$this->addFlashMessage(__('Rule updated', 'wp-dxp'), "success");

				$redirectUrl = site_url(WP_DXP_ADMIN_DASHBOARD_INDEX_URL);
				wp_redirect($redirectUrl);
				exit;
			}

			return;
		}

		$this->addFlashMessage(__('An error occurred, please reload the page', 'wp-dxp'), "error");
	}

	/**
	 * Delete action
	 */
	public function deleteAction()
	{
		$id = $this->request->get('id', false);

		$rule = Wp_Dxp_Rule::find($id);

		if (!$rule) {
			$this->showError(__('Rule could not be found', 'wp-dxp'));
		}

		if (!$rule->can_delete) {
			$this->showError(__('Rule cannot be deleted', 'wp-dxp'));
		}

		Wp_Dxp_Rule::delete($id);

		$this->addFlashMessage(__('Rule deleted', 'wp-dxp'), "success");

		$redirectUrl = site_url(WP_DXP_ADMIN_DASHBOARD_INDEX_URL);
		wp_redirect($redirectUrl);
		exit;
	}

	/**
	 * Duplicate action
	 */
	public function duplicateAction()
	{
		$id = $this->request->get('id', false);

		$rule = Wp_Dxp_Rule::find($id);

		if (!$rule) {
			$this->showError(__('Rule could not be found', 'wp-dxp'));
		}

		$duplicateRule = $rule->clone();

		$this->addFlashMessage(__('Rule duplicated', 'wp-dxp'), "success");

		$redirectUrl = site_url(WP_DXP_ADMIN_DASHBOARD_INDEX_URL);
		wp_redirect($redirectUrl);
		exit;
	}

	/**
	 * Save rule to DB (either create or update)
	 * @param  array $data
	 * @param  integer $id
	 * @return Wp_Dxp_Rule
	 */
	protected function saveRule($data, $id = null)
	{
		if ($id) {
			$rule = Wp_Dxp_Rule::find($id);
		}

		if (empty($rule)) {
			$rule = new Wp_Dxp_Rule();
		}

		$rule->populateFromArray($data);

		// Is new rule
		if (empty($rule->id)) {
			$rule->type = Wp_Dxp_Rule_Types::$CUSTOM;
			$rule->created_by = get_current_user_id();
		}

		$rule->save();

		return $rule;
	}

	/**
	 * Validate data, likely from submitted form
	 * @param  array $data
	 * @param  boolean $existingModel
	 * @return array
	 */
	protected function validate($data, $existingModel = false)
	{
		if (empty($data)) {
			$this->addValidationError('form', __('Form is not valid', 'wp-dxp'));
		} else {
			if ($existingModel) {
				if (empty($data['id'])) {
					$this->addValidationError('form', __('ID is required to update a rule', 'wp-dxp'));
				}
			}

			if (empty($data['name'])) {
				$this->addValidationError('name', __('Name is required', 'wp-dxp'));
			}

			if (empty($data['category_id'])) {
				$this->addValidationError('category_id', __('Category is required', 'wp-dxp'));
			}

			if (!empty($data['conditions']['measure'])) {
				foreach ($data['conditions']['measure'] as $i => $measure) {
					if (empty($measure)) {
						$this->addValidationError('conditions['.$i.']', __('A measure for the condition should be selected', 'wp-dxp'));
					}
				}
			}

			if (!empty($data['conditions']['comparator'])) {
				foreach ($data['conditions']['comparator'] as $i => $comparator) {
					if (empty($comparator)) {
						$this->addValidationError('conditions['.$i.']', __('A comparator for the condition should be selected', 'wp-dxp'));
					}
				}
			}

			if (!empty($data['conditions']['value'])) {
				foreach ($data['conditions']['value'] as $i => $value) {
					if (empty($value)) {
						$this->addValidationError('conditions['.$i.']', __('The value for the condition should not be empty', 'wp-dxp'));
					}
				}
			}
		}
	}
}